==> cbra-0.50.0.tar.gz: cbra-0.50.0/tests/oauth2/tokenmixin.py <==
# pylint: skip-file
import datetime
from lib2to3.pgen2 import token
import secrets
import typing

import ckms
import pytest
import pytest_asyncio
from ckms.jose import PayloadCodec
from ckms.jose import JSONWebToken
from ckms.jose.models import JSONWebKey
from httpx import AsyncClient

from cbra.ext.oauth2 import GrantType
from cbra.ext.oauth2 import ISubject


class TokenMixin:
    base_url: str = "https://cbra-oauth2.localhost"
    default_scope: typing.Set[str] = {"read"}

    @pytest.fixture
    def token_audience(self) -> str:
        return f'{self.base_url}/token'

    async def get_jwt_grant(self, subject: ISubject, **claims: typing.Any) -> str:
        codec = await self.get_subject_codec(subject)
        return await self.sign_jwt(codec, **claims)

    @pytest_asyncio.fixture # type: ignore
    async def grant(self, current_subject: ISubject, token_audience: str):
        return await self.get_jwt_grant(
            subject=current_subject,
            iss=current_subject.sub,
            aud=token_audience,
            sub=current_subject.sub
        )

    @pytest.fixture
    def scope(self) -> typing.Set[str]:
        return self.default_scope

    @pytest_asyncio.fixture # type: ignore
    async def token(
        self,
        client: AsyncClient,
        grant: str,
        current_subject: ISubject,
        scope: typing.Set[str]
    ):
        current_subject.client_scope['jwt'] = scope
        request = dict(
            client_id='jwt',
            grant_type=GrantType.jwt_bearer,
            assertion=grant
        )
        if scope:
            request['scope'] = str.join(' ', sorted(scope)) 
        response = await client.post(url='/token', json=request)
        assert response.status_code == 200, response.text
        dto = response.json()
        return dto['access_token']

    async def get_subject_codec(self, subject: ISubject) -> PayloadCodec:
        jwk = JSONWebKey.generate('OKP', 'Ed25519')
        keychain = ckms.Keychain()
        keychain.register({
            'sig': {
                'provider': 'local',
                'key': jwk.get_impl()
            }
        })
        await keychain.setup()

        # TODO: This is quite a hack to ensure that the keyset can
        # resolve the key.
        meta = keychain.get_key_metadata('sig')
        jwk.kid = meta.get_signing_kid()
        assert jwk.kid is not None
        assert jwk.kid == jwk.public.kid
        subject.register_public_key(jwk.public)
        # TODO: End hack

        return PayloadCodec(signer=keychain)

    async def sign_jwt(
        self,
        codec: PayloadCodec,
        **claims: typing.Any
    ) -> str:
        now = int(datetime.datetime.utcnow().timestamp())
        defaults = {
            'jti': secrets.token_urlsafe(24),
            'iat': now,
            'exp': now+600,
            'nbf': now
        }
        defaults.update(claims)
        jwt = JSONWebToken(**defaults)
        jws = await codec.encode(jwt).sign('EdDSA', 'sig')
        return bytes.decode(bytes(jws), 'ascii')

==> cbra-0.106.0.tar.gz: cbra-0.106.0/tests/oauth2/tokenmixin.py <==
# pylint: skip-file
# type: ignore
import datetime
import secrets
import typing

import pytest
import pytest_asyncio
from ckms.jose import PayloadCodec
from ckms.types import JSONWebToken
from ckms.types import JSONWebKey
from httpx import AsyncClient

from cbra.ext.oauth2.types import GrantType
from cbra.ext.oauth2.types import ISubject
from cbra.utils import current_timestamp
from .conftest import get_subject_codec

class TokenMixin:
    base_url: str = "https://cbra-oauth2.localhost"
    default_scope: typing.Set[str] = {"read"}

    @pytest.fixture
    def token_audience(self) -> str:
        return f'{self.base_url}/oauth2/token'

    async def get_jwt_grant(self, subject: ISubject, **claims: typing.Any) -> str:
        codec = await get_subject_codec(subject)
        return await self.sign_jwt(codec, **claims)

    @pytest_asyncio.fixture # type: ignore
    async def grant(self, current_subject: ISubject, token_audience: str):
        return await self.get_jwt_grant(
            subject=current_subject,
            iss=current_subject.sub,
            aud=token_audience,
            sub=current_subject.sub
        )

    @pytest.fixture
    def scope(self) -> typing.Set[str]:
        return self.default_scope

    @pytest_asyncio.fixture # type: ignore
    async def token(
        self,
        client: AsyncClient,
        grant: str,
        current_subject: ISubject,
        scope: typing.Set[str]
    ):
        current_subject.client_scope['jwt'] = scope
        request = dict(
            client_id='jwt',
            grant_type=GrantType.jwt_bearer,
            assertion=grant
        )
        if scope:
            request['scope'] = str.join(' ', sorted(scope))
        response = await client.post(url='/oauth2/token', json=request)
        assert response.status_code == 200, response.text
        dto = response.json()
        return dto['access_token']

    async def sign_jwt(
        self,
        codec: PayloadCodec,
        **claims: typing.Any
    ) -> str:
        now = current_timestamp()
        defaults = {
            'jti': secrets.token_urlsafe(24),
            'iat': now,
            'exp': now+600,
            'nbf': now
        }
        defaults.update(claims)
        return await codec.encode(
            JSONWebToken(**defaults),
            signers=['sig']
        )
